@@ -25,6 +25,7 @@
 #include "config.h"
 
 #include <assert.h>
+#include <errno.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,6 +38,7 @@
 
 #include <pango/pangocairo.h>
 
+#include "shared/config-parser.h"
 #include "shared/helpers.h"
 #include "shared/xalloc.h"
 #include "window.h"
@@ -719,6 +721,7 @@ text_entry_destroy(struct text_entry *entry)
 	zwp_text_input_v1_destroy(entry->text_input);
 	g_clear_object(&entry->layout);
 	free(entry->text);
+	free(entry->preferred_language);
 	free(entry);
 }
 
@@ -1488,39 +1491,123 @@ global_handler(struct display *display, uint32_t name,
 	}
 }
 
+/** Display help for command line options, and exit */
+static uint32_t opt_help = 0;
+
+/** Require a distinct click to show the input panel (virtual keyboard) */
+static uint32_t opt_click_to_show = 0;
+
+/** Set a specific (RFC-3066) language.  Used for the virtual keyboard, etc. */
+static const char *opt_preferred_language = NULL;
+
+/**
+ * \brief command line options for editor
+ */
+static const struct weston_option editor_options[] = {
+	{ WESTON_OPTION_BOOLEAN, "help", 'h', &opt_help },
+	{ WESTON_OPTION_BOOLEAN, "click-to-show", 'C', &opt_click_to_show },
+	{ WESTON_OPTION_STRING, "preferred-language", 'L', &opt_preferred_language },
+};
+
+static void
+usage(const char *program_name, int exit_code)
+{
+	unsigned k;
+
+	fprintf(stderr, "Usage: %s [OPTIONS] [FILENAME]\n\n", program_name);
+	for (k = 0; k < ARRAY_LENGTH(editor_options); k++) {
+		const struct weston_option *p = &editor_options[k];
+		if (p->name) {
+			fprintf(stderr, "  --%s", p->name);
+			if (p->type != WESTON_OPTION_BOOLEAN)
+				fprintf(stderr, "=VALUE");
+			fprintf(stderr, "\n");
+		}
+		if (p->short_name) {
+			fprintf(stderr, "  -%c", p->short_name);
+			if (p->type != WESTON_OPTION_BOOLEAN)
+				fprintf(stderr, "VALUE");
+			fprintf(stderr, "\n");
+		}
+	}
+	exit(exit_code);
+}
+
+/* Load the contents of a file into a UTF-8 text buffer and return it.
+ *
+ * Caller is responsible for freeing the buffer when done.
+ * On error, returns NULL.
+ */
+static char *
+read_file(char *filename)
+{
+	char *buffer = NULL;
+	int buf_size, read_size;
+	FILE *fin;
+	int errsv;
+
+	fin = fopen(filename, "r");
+	if (fin == NULL)
+		goto error;
+
+	/* Determine required buffer size */
+	if (fseek(fin, 0, SEEK_END) != 0)
+		goto error;
+	buf_size = ftell(fin);
+	if (buf_size < 0)
+		goto error;
+	rewind(fin);
+
+	/* Create buffer and read in the text */
+	buffer = (char*) malloc(sizeof(char) * (buf_size + 1));
+	if (buffer == NULL)
+		goto error;
+	read_size = fread(buffer, sizeof(char), buf_size, fin);
+	fclose(fin);
+	if (buf_size != read_size)
+		goto error;
+	buffer[buf_size] = '\0';
+
+	return buffer;
+
+error:
+	errsv = errno;
+	if (fin)
+		fclose(fin);
+	free(buffer);
+	errno = errsv ? errsv : EINVAL;
+
+	return NULL;
+}
+
 int
 main(int argc, char *argv[])
 {
 	struct editor editor;
-	int i;
-	uint32_t click_to_show = 0;
-	const char *preferred_language = NULL;
+	char *text_buffer = NULL;
 
-	for (i = 1; i < argc; i++) {
-		if (strcmp("--click-to-show", argv[i]) == 0)
-			click_to_show = 1;
-		else if (strcmp("--preferred-language", argv[i]) == 0 &&
-			 i + 1 < argc) {
-			preferred_language = argv[i + 1];
-			i++;
-		} else {
-			printf("Usage: %s [OPTIONS]\n"
-			       "  --click-to-show\n"
-			       "  --preferred-language LANGUAGE\n",
-			       argv[0]);
-			return 1;
+	parse_options(editor_options, ARRAY_LENGTH(editor_options),
+		      &argc, argv);
+	if (opt_help)
+		usage(argv[0], EXIT_SUCCESS);
+
+	if (argc > 1) {
+		if (argv[1][0] == '-')
+			usage(argv[0], EXIT_FAILURE);
+
+		text_buffer = read_file(argv[1]);
+		if (text_buffer == NULL) {
+			fprintf(stderr, "could not read file '%s': %m\n", argv[1]);
+			return -1;
 		}
 	}
 
 	memset(&editor, 0, sizeof editor);
 
-#ifdef HAVE_PANGO
-	g_type_init();
-#endif
-
 	editor.display = display_create(&argc, argv);
 	if (editor.display == NULL) {
 		fprintf(stderr, "failed to create display: %m\n");
+		free(text_buffer);
 		return -1;
 	}
 
@@ -1529,19 +1616,24 @@ main(int argc, char *argv[])
 
 	if (editor.text_input_manager == NULL) {
 		fprintf(stderr, "No text input manager global\n");
+		display_destroy(editor.display);
+		free(text_buffer);
 		return -1;
 	}
 
 	editor.window = window_create(editor.display);
 	editor.widget = window_frame_create(editor.window, &editor);
 
-	editor.entry = text_entry_create(&editor, "Entry");
-	editor.entry->click_to_show = click_to_show;
-	if (preferred_language)
-		editor.entry->preferred_language = strdup(preferred_language);
+	if (text_buffer)
+		editor.entry = text_entry_create(&editor, text_buffer);
+	else
+		editor.entry = text_entry_create(&editor, "Entry");
+	editor.entry->click_to_show = opt_click_to_show;
+	if (opt_preferred_language)
+		editor.entry->preferred_language = strdup(opt_preferred_language);
 	editor.editor = text_entry_create(&editor, "Numeric");
 	editor.editor->content_purpose = ZWP_TEXT_INPUT_V1_CONTENT_PURPOSE_NUMBER;
-	editor.editor->click_to_show = click_to_show;
+	editor.editor->click_to_show = opt_click_to_show;
 	editor.selection = NULL;
 	editor.selected_text = NULL;
 
@@ -1569,6 +1661,7 @@ main(int argc, char *argv[])
 	widget_destroy(editor.widget);
 	window_destroy(editor.window);
 	display_destroy(editor.display);
+	free(text_buffer);
 
 	return 0;
 }
