@@ -333,6 +333,8 @@ nested_client_destroy(struct nested_client *client)
 		       EGL_NO_SURFACE, EGL_NO_SURFACE,
 		       EGL_NO_CONTEXT);
 
+	weston_platform_destroy_egl_surface(client->egl_display,
+					    client->egl_surface);
 	wl_egl_window_destroy(client->native);
 
 	wl_surface_destroy(client->surface);
@@ -343,6 +345,8 @@ nested_client_destroy(struct nested_client *client)
 	wl_registry_destroy(client->registry);
 	wl_display_flush(client->display);
 	wl_display_disconnect(client->display);
+	eglTerminate(client->egl_display);
+	eglReleaseThread();
 }
 
 int
