@@ -32,6 +32,7 @@
 
 #include <linux/input.h>
 #include <cairo.h>
+#include <wayland-util.h>
 
 #include "shared/helpers.h"
 #include "window.h"
@@ -184,7 +185,7 @@ fullscreen_handler(struct window *window, void *data)
 
 static void
 draw_string(cairo_t *cr,
-            const char *fmt, ...) __attribute__((format (gnu_printf, 2, 3)));
+            const char *fmt, ...) WL_PRINTF(2, 3);
 
 static void
 draw_string(cairo_t *cr,
@@ -287,10 +288,6 @@ main(int argc, char *argv[])
 
 	memset(&stacking, 0, sizeof stacking);
 
-#ifdef HAVE_PANGO
-	g_type_init();
-#endif
-
 	stacking.display = display_create(&argc, argv);
 	if (stacking.display == NULL) {
 		fprintf(stderr, "Failed to create display: %m\n");
