@@ -3,23 +3,24 @@
  * Copyright © 2011 Benjamin Franzke
  * Copyright © 2012-2013 Collabora, Ltd.
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include "config.h"
@@ -40,6 +41,9 @@
 #include <EGL/egl.h>
 #include <EGL/eglext.h>
 
+#include "shared/helpers.h"
+#include "shared/xalloc.h"
+#include "shared/zalloc.h"
 #include "window.h"
 
 #if 0
@@ -213,7 +217,7 @@ egl_state_create(struct wl_display *display)
 	EGLint major, minor, n;
 	EGLBoolean ret;
 
-	egl = calloc(1, sizeof *egl);
+	egl = zalloc(sizeof *egl);
 	assert(egl);
 
 	egl->dpy =
@@ -496,8 +500,7 @@ triangle_create(struct window *window, struct egl_state *egl)
 {
 	struct triangle *tri;
 
-	tri = xmalloc(sizeof *tri);
-	memset(tri, 0, sizeof *tri);
+	tri = xzalloc(sizeof *tri);
 
 	tri->egl = egl;
 	tri->widget = window_add_subsurface(window, tri,
@@ -515,7 +518,8 @@ static void
 triangle_destroy(struct triangle *tri)
 {
 	if (tri->egl_surface)
-		eglDestroySurface(tri->egl->dpy, tri->egl_surface);
+		weston_platform_destroy_egl_surface(tri->egl->dpy,
+						    tri->egl_surface);
 
 	if (tri->egl_window)
 		wl_egl_window_destroy(tri->egl_window);
@@ -718,8 +722,7 @@ demoapp_create(struct display *display)
 {
 	struct demoapp *app;
 
-	app = xmalloc(sizeof *app);
-	memset(app, 0, sizeof *app);
+	app = xzalloc(sizeof *app);
 
 	app->egl = egl_state_create(display_get_display(display));
 
