@@ -115,17 +115,6 @@ move_client(struct client *client, int x, int y)
 	frame_callback_wait(client, &done);
 }
 
-int
-get_n_egl_buffers(struct client *client)
-{
-	client->test->n_egl_buffers = -1;
-
-	weston_test_get_n_egl_buffers(client->test->weston_test);
-	wl_display_roundtrip(client->wl_display);
-
-	return client->test->n_egl_buffers;
-}
-
 static void
 pointer_handle_enter(void *data, struct wl_pointer *wl_pointer,
 		     uint32_t serial, struct wl_surface *wl_surface,
@@ -516,14 +505,6 @@ test_handle_pointer_position(void *data, struct weston_test *weston_test,
 		test->pointer_x, test->pointer_y);
 }
 
-static void
-test_handle_n_egl_buffers(void *data, struct weston_test *weston_test, uint32_t n)
-{
-	struct test *test = data;
-
-	test->n_egl_buffers = n;
-}
-
 static void
 test_handle_capture_screenshot_done(void *data, struct weston_test *weston_test)
 {
@@ -535,7 +516,6 @@ test_handle_capture_screenshot_done(void *data, struct weston_test *weston_test)
 
 static const struct weston_test_listener test_listener = {
 	test_handle_pointer_position,
-	test_handle_n_egl_buffers,
 	test_handle_capture_screenshot_done,
 };
 
@@ -932,9 +912,10 @@ output_path(void)
 	char *path = getenv("WESTON_TEST_OUTPUT_PATH");
 
 	if (!path)
-		return ".";
+		return "./logs";
+
 	return path;
-	}
+}
 
 char*
 screenshot_output_filename(const char *basename, uint32_t seq)
