@@ -25,6 +25,7 @@
 
 #include "config.h"
 
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -117,7 +118,7 @@ weston_wm_get_incr_chunk(struct weston_wm *wm)
 	dump_property(wm, wm->atom.wl_selection, reply);
 
 	if (xcb_get_property_value_length(reply) > 0) {
-		/* reply's ownership is transfered to wm, which is responsible
+		/* reply's ownership is transferred to wm, which is responsible
 		 * for freeing it */
 		weston_wm_write_property(wm, reply);
 	} else {
@@ -197,7 +198,7 @@ weston_wm_get_selection_targets(struct weston_wm *wm)
 		return;
 	}
 
-	source = malloc(sizeof *source);
+	source = zalloc(sizeof *source);
 	if (source == NULL) {
 		free(reply);
 		return;
@@ -251,7 +252,7 @@ weston_wm_get_selection_data(struct weston_wm *wm)
 		free(reply);
 	} else {
 		wm->incr = 0;
-		/* reply's ownership is transfered to wm, which is responsible
+		/* reply's ownership is transferred to wm, which is responsible
 		 * for freeing it */
 		weston_wm_write_property(wm, reply);
 	}
@@ -405,7 +406,7 @@ weston_wm_read_data_source(int fd, uint32_t mask, void *data)
 			wm->property_source = NULL;
 		} else {
 			weston_log("got %zu bytes, "
-				"property deleted, seting new property\n",
+				"property deleted, setting new property\n",
 				wm->source_data.size);
 			weston_wm_flush_source_data(wm);
 		}
@@ -429,7 +430,7 @@ weston_wm_read_data_source(int fd, uint32_t mask, void *data)
 				"property delete\n", wm->source_data.size);
 		} else {
 			weston_log("got %zu bytes, "
-				"property deleted, seting new property\n",
+				"property deleted, setting new property\n",
 				wm->source_data.size);
 			weston_wm_flush_source_data(wm);
 		}
@@ -655,8 +656,6 @@ weston_wm_set_selection(struct wl_listener *listener, void *data)
 	struct weston_wm *wm =
 		container_of(listener, struct weston_wm, selection_listener);
 	struct weston_data_source *source = seat->selection_data_source;
-	const char **p, **end;
-	int has_text_plain = 0;
 
 	if (source == NULL) {
 		if (wm->selection_owner == wm->selection_window)
@@ -670,28 +669,10 @@ weston_wm_set_selection(struct wl_listener *listener, void *data)
 	if (source->send == data_source_send)
 		return;
 
-	p = source->mime_types.data;
-	end = (const char **)
-		((char *) source->mime_types.data + source->mime_types.size);
-	while (p < end) {
-		weston_log("  %s\n", *p);
-		if (strcmp(*p, "text/plain") == 0 ||
-		    strcmp(*p, "text/plain;charset=utf-8") == 0)
-			has_text_plain = 1;
-		p++;
-	}
-
-	if (has_text_plain) {
-		xcb_set_selection_owner(wm->conn,
-					wm->selection_window,
-					wm->atom.clipboard,
-					XCB_TIME_CURRENT_TIME);
-	} else {
-		xcb_set_selection_owner(wm->conn,
-					XCB_ATOM_NONE,
-					wm->atom.clipboard,
-					XCB_TIME_CURRENT_TIME);
-	}
+	xcb_set_selection_owner(wm->conn,
+				wm->selection_window,
+				wm->atom.clipboard,
+				XCB_TIME_CURRENT_TIME);
 }
 
 void
@@ -728,6 +709,8 @@ weston_wm_selection_init(struct weston_wm *wm)
 					  wm->atom.clipboard, mask);
 
 	seat = weston_wm_pick_seat(wm);
+	if (seat == NULL)
+		return;
 	wm->selection_listener.notify = weston_wm_set_selection;
 	wl_signal_add(&seat->selection_signal, &wm->selection_listener);
 
