@@ -35,14 +35,16 @@
 #include <errno.h>
 #include <unistd.h>
 #include <signal.h>
+#include <limits.h>
+#include <assert.h>
 #include <X11/Xcursor/Xcursor.h>
 #include <linux/input.h>
 
+#include "compositor.h"
 #include "xwayland.h"
 #include "xwayland-internal-interface.h"
 
 #include "cairo-util.h"
-#include "compositor.h"
 #include "hash.h"
 #include "shared/helpers.h"
 
@@ -125,6 +127,11 @@ struct motif_wm_hints {
 #define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10   /* move via keyboard */
 #define _NET_WM_MOVERESIZE_CANCEL           11   /* cancel operation */
 
+struct weston_output_weak_ref {
+	struct weston_output *output;
+	struct wl_listener destroy_listener;
+};
+
 struct weston_wm_window {
 	struct weston_wm *wm;
 	xcb_window_t id;
@@ -146,8 +153,12 @@ struct weston_wm_window {
 	uint32_t protocols;
 	xcb_atom_t type;
 	int width, height;
-	int x, y;
+	int x;
+	int y;
 	bool pos_dirty;
+	int map_request_x;
+	int map_request_y;
+	struct weston_output_weak_ref legacy_fullscreen_output;
 	int saved_width, saved_height;
 	int decorate;
 	int override_redirect;
@@ -170,6 +181,11 @@ weston_wm_set_net_active_window(struct weston_wm *wm, xcb_window_t window);
 static void
 weston_wm_window_schedule_repaint(struct weston_wm_window *window);
 
+static int
+legacy_fullscreen(struct weston_wm *wm,
+		  struct weston_wm_window *window,
+		  struct weston_output **output_ret);
+
 static void
 xserver_map_shell_surface(struct weston_wm_window *window,
 			  struct weston_surface *surface);
@@ -208,6 +224,47 @@ wm_log_continue(const char *fmt, ...)
 #endif
 }
 
+static void
+weston_output_weak_ref_init(struct weston_output_weak_ref *ref)
+{
+	ref->output = NULL;
+}
+
+static void
+weston_output_weak_ref_clear(struct weston_output_weak_ref *ref)
+{
+	if (!ref->output)
+		return;
+
+	wl_list_remove(&ref->destroy_listener.link);
+	ref->output = NULL;
+}
+
+static void
+weston_output_weak_ref_handle_destroy(struct wl_listener *listener, void *data)
+{
+	struct weston_output_weak_ref *ref;
+
+	ref = wl_container_of(listener, ref, destroy_listener);
+	assert(ref->output == data);
+
+	weston_output_weak_ref_clear(ref);
+}
+
+static void
+weston_output_weak_ref_set(struct weston_output_weak_ref *ref,
+			   struct weston_output *output)
+{
+	weston_output_weak_ref_clear(ref);
+
+	if (!output)
+		return;
+
+	ref->destroy_listener.notify = weston_output_weak_ref_handle_destroy;
+	wl_signal_add(&output->destroy_signal, &ref->destroy_listener);
+	ref->output = output;
+}
+
 static bool __attribute__ ((warn_unused_result))
 wm_lookup_window(struct weston_wm *wm, xcb_window_t hash,
 		 struct weston_wm_window **window)
@@ -386,7 +443,10 @@ read_and_dump_property(struct weston_wm *wm,
 	free(reply);
 }
 
-/* We reuse some predefined, but otherwise useles atoms */
+/* We reuse some predefined, but otherwise useles atoms
+ * as local type placeholders that never touch the X11 server,
+ * to make weston_wm_window_read_properties() less exceptional.
+ */
 #define TYPE_WM_PROTOCOLS	XCB_ATOM_CUT_BUFFER0
 #define TYPE_MOTIF_WM_HINTS	XCB_ATOM_CUT_BUFFER1
 #define TYPE_NET_WM_STATE	XCB_ATOM_CUT_BUFFER2
@@ -396,26 +456,24 @@ static void
 weston_wm_window_read_properties(struct weston_wm_window *window)
 {
 	struct weston_wm *wm = window->wm;
-	const struct weston_desktop_xwayland_interface *xwayland_interface =
-		wm->server->compositor->xwayland_interface;
 
-#define F(field) offsetof(struct weston_wm_window, field)
+#define F(field) (&window->field)
 	const struct {
 		xcb_atom_t atom;
 		xcb_atom_t type;
-		int offset;
+		void *ptr;
 	} props[] = {
-		{ XCB_ATOM_WM_CLASS, XCB_ATOM_STRING, F(class) },
-		{ XCB_ATOM_WM_NAME, XCB_ATOM_STRING, F(name) },
-		{ XCB_ATOM_WM_TRANSIENT_FOR, XCB_ATOM_WINDOW, F(transient_for) },
-		{ wm->atom.wm_protocols, TYPE_WM_PROTOCOLS, F(protocols) },
-		{ wm->atom.wm_normal_hints, TYPE_WM_NORMAL_HINTS, F(protocols) },
-		{ wm->atom.net_wm_state, TYPE_NET_WM_STATE },
-		{ wm->atom.net_wm_window_type, XCB_ATOM_ATOM, F(type) },
-		{ wm->atom.net_wm_name, XCB_ATOM_STRING, F(name) },
-		{ wm->atom.net_wm_pid, XCB_ATOM_CARDINAL, F(pid) },
-		{ wm->atom.motif_wm_hints, TYPE_MOTIF_WM_HINTS, 0 },
-		{ wm->atom.wm_client_machine, XCB_ATOM_WM_CLIENT_MACHINE, F(machine) },
+		{ XCB_ATOM_WM_CLASS,           XCB_ATOM_STRING,            F(class) },
+		{ XCB_ATOM_WM_NAME,            XCB_ATOM_STRING,            F(name) },
+		{ XCB_ATOM_WM_TRANSIENT_FOR,   XCB_ATOM_WINDOW,            F(transient_for) },
+		{ wm->atom.wm_protocols,       TYPE_WM_PROTOCOLS,          NULL },
+		{ wm->atom.wm_normal_hints,    TYPE_WM_NORMAL_HINTS,       NULL },
+		{ wm->atom.net_wm_state,       TYPE_NET_WM_STATE,          NULL },
+		{ wm->atom.net_wm_window_type, XCB_ATOM_ATOM,              F(type) },
+		{ wm->atom.net_wm_name,        XCB_ATOM_STRING,            F(name) },
+		{ wm->atom.net_wm_pid,         XCB_ATOM_CARDINAL,          F(pid) },
+		{ wm->atom.motif_wm_hints,     TYPE_MOTIF_WM_HINTS,        NULL },
+		{ wm->atom.wm_client_machine,  XCB_ATOM_WM_CLIENT_MACHINE, F(machine) },
 	};
 #undef F
 
@@ -454,7 +512,7 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 			continue;
 		}
 
-		p = ((char *) window + props[i].offset);
+		p = props[i].ptr;
 
 		switch (props[i].type) {
 		case XCB_ATOM_WM_CLIENT_MACHINE:
@@ -539,15 +597,13 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 		if (!window->machine || strcmp(window->machine, name))
 			window->pid = 0;
 	}
-
-	if (window->shsurf && window->name)
-		xwayland_interface->set_title(window->shsurf, window->name);
-	if (window->frame && window->name)
-		frame_set_title(window->frame, window->name);
-	if (window->shsurf && window->pid > 0)
-		xwayland_interface->set_pid(window->shsurf, window->pid);
 }
 
+#undef TYPE_WM_PROTOCOLS
+#undef TYPE_MOTIF_WM_HINTS
+#undef TYPE_NET_WM_STATE
+#undef TYPE_WM_NORMAL_HINTS
+
 static void
 weston_wm_window_get_frame_size(struct weston_wm_window *window,
 				int *width, int *height)
@@ -683,12 +739,15 @@ weston_wm_handle_configure_notify(struct weston_wm *wm, xcb_generic_event_t *eve
 {
 	xcb_configure_notify_event_t *configure_notify =
 		(xcb_configure_notify_event_t *) event;
+	const struct weston_desktop_xwayland_interface *xwayland_api =
+		wm->server->compositor->xwayland_interface;
 	struct weston_wm_window *window;
 
-	wm_log("XCB_CONFIGURE_NOTIFY (window %d) %d,%d @ %dx%d\n",
+	wm_log("XCB_CONFIGURE_NOTIFY (window %d) %d,%d @ %dx%d%s\n",
 	       configure_notify->window,
 	       configure_notify->x, configure_notify->y,
-	       configure_notify->width, configure_notify->height);
+	       configure_notify->width, configure_notify->height,
+	       configure_notify->override_redirect ? ", override" : "");
 
 	if (!wm_lookup_window(wm, configure_notify->window, &window))
 		return;
@@ -703,6 +762,13 @@ weston_wm_handle_configure_notify(struct weston_wm *wm, xcb_generic_event_t *eve
 		if (window->frame)
 			frame_resize_inside(window->frame,
 					    window->width, window->height);
+
+		/* We should check if shsurf has been created because sometimes
+		 * there are races
+		 * (configure_notify is sent before xserver_map_surface) */
+		if (window->shsurf)
+			xwayland_api->set_xwayland(window->shsurf,
+						   window->x, window->y);
 	}
 }
 
@@ -730,6 +796,8 @@ weston_wm_create_surface(struct wl_listener *listener, void *data)
 	if (wl_resource_get_client(surface->resource) != wm->server->client)
 		return;
 
+	wm_log("XWM: create weston_surface %p\n", surface);
+
 	wl_list_for_each(window, &wm->unpaired_window_list, link)
 		if (window->surface_id ==
 		    wl_resource_get_id(surface->resource)) {
@@ -814,6 +882,37 @@ weston_wm_window_activate(struct wl_listener *listener, void *data)
 
 }
 
+/** Control Xwayland wl_surface.commit behaviour
+ *
+ * This function sets the "_XWAYLAND_ALLOW_COMMITS" property of the frame window
+ * (not the content window!) to \p allow.
+ *
+ * If the property is set to \c true, Xwayland will commit whenever it likes.
+ * If the property is set to \c false, Xwayland will not commit.
+ * If the property is not set at all, Xwayland assumes it is \c true.
+ *
+ * \param window The XWM window to control.
+ * \param allow Whether Xwayland is allowed to wl_surface.commit for the window.
+ */
+static void
+weston_wm_window_set_allow_commits(struct weston_wm_window *window, bool allow)
+{
+	struct weston_wm *wm = window->wm;
+	uint32_t property[1];
+
+	assert(window->frame_id != XCB_WINDOW_NONE);
+
+	property[0] = allow ? 1 : 0;
+
+	xcb_change_property(wm->conn,
+			    XCB_PROP_MODE_REPLACE,
+			    window->frame_id,
+			    wm->atom.allow_commits,
+			    XCB_ATOM_CARDINAL,
+			    32, /* format */
+			    1, property);
+}
+
 #define ICCCM_WITHDRAWN_STATE	0
 #define ICCCM_NORMAL_STATE	1
 #define ICCCM_ICONIC_STATE	3
@@ -951,6 +1050,7 @@ weston_wm_handle_map_request(struct weston_wm *wm, xcb_generic_event_t *event)
 	xcb_map_request_event_t *map_request =
 		(xcb_map_request_event_t *) event;
 	struct weston_wm_window *window;
+	struct weston_output *output;
 
 	if (our_resource(wm, map_request->window)) {
 		wm_log("XCB_MAP_REQUEST (window %d, ours)\n",
@@ -963,18 +1063,49 @@ weston_wm_handle_map_request(struct weston_wm *wm, xcb_generic_event_t *event)
 
 	weston_wm_window_read_properties(window);
 
+	/* For a new Window, MapRequest happens before the Window is realized
+	 * in Xwayland. We do the real xcb_map_window() here as a response to
+	 * MapRequest. The Window will get realized (wl_surface created in
+	 * Wayland and WL_SURFACE_ID sent in X11) when it has been mapped for
+	 * real.
+	 *
+	 * MapRequest only happens for (X11) unmapped Windows. On UnmapNotify,
+	 * we reset shsurf to NULL, so even if X11 connection races far ahead
+	 * of the Wayland connection and the X11 client is repeatedly mapping
+	 * and unmapping, we will never have shsurf set on MapRequest.
+	 */
+	assert(!window->shsurf);
+
+	window->map_request_x = window->x;
+	window->map_request_y = window->y;
+
 	if (window->frame_id == XCB_WINDOW_NONE)
-		weston_wm_window_create_frame(window);
+		weston_wm_window_create_frame(window); /* sets frame_id */
+	assert(window->frame_id != XCB_WINDOW_NONE);
 
-	wm_log("XCB_MAP_REQUEST (window %d, %p, frame %d)\n",
-	       window->id, window, window->frame_id);
+	wm_log("XCB_MAP_REQUEST (window %d, %p, frame %d, %dx%d @ %d,%d)\n",
+	       window->id, window, window->frame_id,
+	       window->width, window->height,
+	       window->map_request_x, window->map_request_y);
 
+	weston_wm_window_set_allow_commits(window, false);
 	weston_wm_window_set_wm_state(window, ICCCM_NORMAL_STATE);
 	weston_wm_window_set_net_wm_state(window);
 	weston_wm_window_set_virtual_desktop(window, 0);
 
+	if (legacy_fullscreen(wm, window, &output)) {
+		window->fullscreen = 1;
+		weston_output_weak_ref_set(&window->legacy_fullscreen_output,
+					   output);
+	}
+
 	xcb_map_window(wm->conn, map_request->window);
 	xcb_map_window(wm->conn, window->frame_id);
+
+	/* Mapped in the X server, we can draw immediately.
+	 * Cannot set pending state though, no weston_surface until
+	 * xserver_map_shell_surface() time. */
+	weston_wm_window_schedule_repaint(window);
 }
 
 static void
@@ -988,7 +1119,8 @@ weston_wm_handle_map_notify(struct weston_wm *wm, xcb_generic_event_t *event)
 			return;
 	}
 
-	wm_log("XCB_MAP_NOTIFY (window %d)\n", map_notify->window);
+	wm_log("XCB_MAP_NOTIFY (window %d%s)\n", map_notify->window,
+	       map_notify->override_redirect ? ", override" : "");
 }
 
 static void
@@ -1036,25 +1168,14 @@ weston_wm_handle_unmap_notify(struct weston_wm *wm, xcb_generic_event_t *event)
 }
 
 static void
-weston_wm_window_draw_decoration(void *data)
+weston_wm_window_draw_decoration(struct weston_wm_window *window)
 {
-	struct weston_wm_window *window = data;
-	struct weston_wm *wm = window->wm;
-	struct theme *t = wm->theme;
 	cairo_t *cr;
-	int x, y, width, height;
-	int32_t input_x, input_y, input_w, input_h;
-	const struct weston_desktop_xwayland_interface *xwayland_interface =
-		wm->server->compositor->xwayland_interface;
-	uint32_t flags = 0;
-	struct weston_view *view;
+	int width, height;
 
-	weston_wm_window_read_properties(window);
-
-	window->repaint_source = NULL;
+	wm_log("XWM: draw decoration, win %d\n", window->id);
 
 	weston_wm_window_get_frame_size(window, &width, &height);
-	weston_wm_window_get_child_position(window, &x, &y);
 
 	cairo_xcb_surface_set_size(window->cairo_surface, width, height);
 	cr = cairo_create(window->cairo_surface);
@@ -1062,53 +1183,106 @@ weston_wm_window_draw_decoration(void *data)
 	if (window->fullscreen) {
 		/* nothing */
 	} else if (window->decorate) {
-		if (wm->focus_window == window)
-			flags |= THEME_FRAME_ACTIVE;
-
+		frame_set_title(window->frame, window->name);
 		frame_repaint(window->frame, cr);
 	} else {
 		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
 		cairo_set_source_rgba(cr, 0, 0, 0, 0);
 		cairo_paint(cr);
 
-		render_shadow(cr, t->shadow, 2, 2, width + 8, height + 8, 64, 64);
+		render_shadow(cr, window->wm->theme->shadow,
+			      2, 2, width + 8, height + 8, 64, 64);
 	}
 
 	cairo_destroy(cr);
+	cairo_surface_flush(window->cairo_surface);
+	xcb_flush(window->wm->conn);
+}
 
-	if (window->surface) {
-		pixman_region32_fini(&window->surface->pending.opaque);
-		if (window->has_alpha) {
-			pixman_region32_init(&window->surface->pending.opaque);
-		} else {
-			/* We leave an extra pixel around the X window area to
-			 * make sure we don't sample from the undefined alpha
-			 * channel when filtering. */
-			pixman_region32_init_rect(&window->surface->pending.opaque,
-						  x - 1, y - 1,
-						  window->width + 2,
-						  window->height + 2);
-		}
-		wl_list_for_each(view, &window->surface->views, surface_link)
-			weston_view_geometry_dirty(view);
+static void
+weston_wm_window_set_pending_state(struct weston_wm_window *window)
+{
+	int x, y, width, height;
+	int32_t input_x, input_y, input_w, input_h;
+	const struct weston_desktop_xwayland_interface *xwayland_interface =
+		window->wm->server->compositor->xwayland_interface;
 
-		pixman_region32_fini(&window->surface->pending.input);
+	if (!window->surface)
+		return;
 
-		if (window->decorate && !window->fullscreen) {
-			frame_input_rect(window->frame, &input_x, &input_y,
-					 &input_w, &input_h);
-		} else {
-			input_x = x;
-			input_y = y;
-			input_w = width;
-			input_h = height;
-		}
+	weston_wm_window_get_frame_size(window, &width, &height);
+	weston_wm_window_get_child_position(window, &x, &y);
 
-		pixman_region32_init_rect(&window->surface->pending.input,
-					  input_x, input_y, input_w, input_h);
+	pixman_region32_fini(&window->surface->pending.opaque);
+	if (window->has_alpha) {
+		pixman_region32_init(&window->surface->pending.opaque);
+	} else {
+		/* We leave an extra pixel around the X window area to
+		 * make sure we don't sample from the undefined alpha
+		 * channel when filtering. */
+		pixman_region32_init_rect(&window->surface->pending.opaque,
+					  x - 1, y - 1,
+					  window->width + 2,
+					  window->height + 2);
+	}
 
-		xwayland_interface->set_window_geometry(window->shsurf,
-							input_x, input_y, input_w, input_h);
+	if (window->decorate && !window->fullscreen) {
+		frame_input_rect(window->frame, &input_x, &input_y,
+				 &input_w, &input_h);
+	} else {
+		input_x = x;
+		input_y = y;
+		input_w = width;
+		input_h = height;
+	}
+
+	wm_log("XWM: win %d geometry: %d,%d %dx%d\n",
+	       window->id, input_x, input_y, input_w, input_h);
+
+	pixman_region32_fini(&window->surface->pending.input);
+	pixman_region32_init_rect(&window->surface->pending.input,
+				  input_x, input_y, input_w, input_h);
+
+	xwayland_interface->set_window_geometry(window->shsurf,
+						input_x, input_y,
+						input_w, input_h);
+	if (window->name)
+		xwayland_interface->set_title(window->shsurf, window->name);
+	if (window->pid > 0)
+		xwayland_interface->set_pid(window->shsurf, window->pid);
+}
+
+static void
+weston_wm_window_do_repaint(void *data)
+{
+	struct weston_wm_window *window = data;
+
+	window->repaint_source = NULL;
+
+	weston_wm_window_read_properties(window);
+
+	weston_wm_window_draw_decoration(window);
+	weston_wm_window_set_pending_state(window);
+}
+
+static void
+weston_wm_window_set_pending_state_OR(struct weston_wm_window *window)
+{
+	int width, height;
+
+	/* for override-redirect windows */
+	assert(window->frame_id == XCB_WINDOW_NONE);
+
+	if (!window->surface)
+		return;
+
+	weston_wm_window_get_frame_size(window, &width, &height);
+	pixman_region32_fini(&window->surface->pending.opaque);
+	if (window->has_alpha) {
+		pixman_region32_init(&window->surface->pending.opaque);
+	} else {
+		pixman_region32_init_rect(&window->surface->pending.opaque, 0, 0,
+					  width, height);
 	}
 }
 
@@ -1116,32 +1290,25 @@ static void
 weston_wm_window_schedule_repaint(struct weston_wm_window *window)
 {
 	struct weston_wm *wm = window->wm;
-	struct weston_view *view;
-	int width, height;
 
 	if (window->frame_id == XCB_WINDOW_NONE) {
-		if (window->surface != NULL) {
-			weston_wm_window_get_frame_size(window, &width, &height);
-			pixman_region32_fini(&window->surface->pending.opaque);
-			if (window->has_alpha) {
-				pixman_region32_init(&window->surface->pending.opaque);
-			} else {
-				pixman_region32_init_rect(&window->surface->pending.opaque, 0, 0,
-							  width, height);
-			}
-			wl_list_for_each(view, &window->surface->views, surface_link)
-				weston_view_geometry_dirty(view);
-		}
+		/* Override-redirect windows go through here, but we
+		 * cannot assert(window->override_redirect); because
+		 * we do not deal with changing OR flag yet.
+		 * XXX: handle OR flag changes in message handlers
+		 */
+		weston_wm_window_set_pending_state_OR(window);
 		return;
 	}
 
 	if (window->repaint_source)
 		return;
 
+	wm_log("XWM: schedule repaint, win %d\n", window->id);
+
 	window->repaint_source =
 		wl_event_loop_add_idle(wm->server->loop,
-				       weston_wm_window_draw_decoration,
-				       window);
+				       weston_wm_window_do_repaint, window);
 }
 
 static void
@@ -1158,7 +1325,8 @@ weston_wm_handle_property_notify(struct weston_wm *wm, xcb_generic_event_t *even
 
 	wm_log("XCB_PROPERTY_NOTIFY: window %d, ", property_notify->window);
 	if (property_notify->state == XCB_PROPERTY_DELETE)
-		wm_log("deleted\n");
+		wm_log_continue("deleted %s\n",
+				get_atom_name(wm->conn, property_notify->atom));
 	else
 		read_and_dump_property(wm, property_notify->window,
 				       property_notify->atom);
@@ -1198,6 +1366,9 @@ weston_wm_window_create(struct weston_wm *wm,
 	window->x = x;
 	window->y = y;
 	window->pos_dirty = false;
+	window->map_request_x = INT_MIN; /* out of range for valid positions */
+	window->map_request_y = INT_MIN; /* out of range for valid positions */
+	weston_output_weak_ref_init(&window->legacy_fullscreen_output);
 
 	geometry_reply = xcb_get_geometry_reply(wm->conn, geometry_cookie, NULL);
 	/* technically we should use XRender and check the visual format's
@@ -1214,6 +1385,8 @@ weston_wm_window_destroy(struct weston_wm_window *window)
 {
 	struct weston_wm *wm = window->wm;
 
+	weston_output_weak_ref_clear(&window->legacy_fullscreen_output);
+
 	if (window->repaint_source)
 		wl_event_source_remove(window->repaint_source);
 	if (window->cairo_surface)
@@ -1244,8 +1417,9 @@ weston_wm_handle_create_notify(struct weston_wm *wm, xcb_generic_event_t *event)
 	xcb_create_notify_event_t *create_notify =
 		(xcb_create_notify_event_t *) event;
 
-	wm_log("XCB_CREATE_NOTIFY (window %d, width %d, height %d%s%s)\n",
+	wm_log("XCB_CREATE_NOTIFY (window %d, at (%d, %d), width %d, height %d%s%s)\n",
 	       create_notify->window,
+	       create_notify->x, create_notify->y,
 	       create_notify->width, create_notify->height,
 	       create_notify->override_redirect ? ", override" : "",
 	       our_resource(wm, create_notify->window) ? ", ours" : "");
@@ -1287,10 +1461,11 @@ weston_wm_handle_reparent_notify(struct weston_wm *wm, xcb_generic_event_t *even
 		(xcb_reparent_notify_event_t *) event;
 	struct weston_wm_window *window;
 
-	wm_log("XCB_REPARENT_NOTIFY (window %d, parent %d, event %d)\n",
+	wm_log("XCB_REPARENT_NOTIFY (window %d, parent %d, event %d%s)\n",
 	       reparent_notify->window,
 	       reparent_notify->parent,
-	       reparent_notify->event);
+	       reparent_notify->event,
+	       reparent_notify->override_redirect ? ", override" : "");
 
 	if (reparent_notify->parent == wm->screen->root) {
 		weston_wm_window_create(wm, reparent_notify->window, 10, 10,
@@ -1306,8 +1481,10 @@ weston_wm_handle_reparent_notify(struct weston_wm *wm, xcb_generic_event_t *even
 struct weston_seat *
 weston_wm_pick_seat(struct weston_wm *wm)
 {
-	return container_of(wm->server->compositor->seat_list.next,
-			    struct weston_seat, link);
+	struct wl_list *seats = wm->server->compositor->seat_list.next;
+	if (wl_list_empty(seats))
+		return NULL;
+	return container_of(seats, struct weston_seat, link);
 }
 
 static struct weston_seat *
@@ -1759,7 +1936,7 @@ weston_wm_handle_button(struct weston_wm *wm, xcb_generic_event_t *event)
 	struct weston_pointer *pointer;
 	struct weston_wm_window *window;
 	enum theme_location location;
-	enum frame_button_state button_state;
+	enum wl_pointer_button_state button_state;
 	uint32_t button_id;
 
 	wm_log("XCB_BUTTON_%s (detail %d)\n",
@@ -1777,7 +1954,8 @@ weston_wm_handle_button(struct weston_wm *wm, xcb_generic_event_t *event)
 	pointer = weston_seat_get_pointer(seat);
 
 	button_state = button->response_type == XCB_BUTTON_PRESS ?
-		FRAME_BUTTON_PRESSED : FRAME_BUTTON_RELEASED;
+		WL_POINTER_BUTTON_STATE_PRESSED :
+		WL_POINTER_BUTTON_STATE_RELEASED;
 	button_id = button->detail == 1 ? BTN_LEFT : BTN_RIGHT;
 
 	/* Make sure we're looking at the right location.  The frame
@@ -1886,6 +2064,12 @@ static void
 weston_wm_handle_focus_in(struct weston_wm *wm, xcb_generic_event_t *event)
 {
 	xcb_focus_in_event_t *focus = (xcb_focus_in_event_t *) event;
+
+	/* Do not interfere with grabs */
+	if (focus->mode == XCB_NOTIFY_MODE_GRAB ||
+	    focus->mode == XCB_NOTIFY_MODE_UNGRAB)
+		return;
+
 	/* Do not let X clients change the focus behind the compositor's
 	 * back. Reset the focus to the old one if it changed. */
 	if (!wm->focus_window || focus->event != wm->focus_window->id)
@@ -2083,6 +2267,7 @@ weston_wm_get_resources(struct weston_wm *wm)
 		{ "XdndFinished",	F(atom.xdnd_finished) },
 		{ "XdndTypeList",	F(atom.xdnd_type_list) },
 		{ "XdndActionCopy",	F(atom.xdnd_action_copy) },
+		{ "_XWAYLAND_ALLOW_COMMITS",	F(atom.allow_commits) },
 		{ "WL_SURFACE_ID",	F(atom.wl_surface_id) }
 	};
 #undef F
@@ -2191,7 +2376,7 @@ weston_wm_create_wm_window(struct weston_wm *wm)
 			    32, /* format */
 			    1, &wm->wm_window);
 
-	/* Claim the WM_S0 selection even though we don't suport
+	/* Claim the WM_S0 selection even though we don't support
 	 * the --replace functionality. */
 	xcb_set_selection_owner(wm->conn,
 				wm->wm_window,
@@ -2487,6 +2672,17 @@ legacy_fullscreen(struct weston_wm *wm,
 	return 0;
 }
 
+static bool
+weston_wm_window_is_positioned(struct weston_wm_window *window)
+{
+	if (window->map_request_x == INT_MIN ||
+	    window->map_request_y == INT_MIN)
+		weston_log("XWM warning: win %d did not see map request\n",
+			   window->id);
+
+	return window->map_request_x != 0 || window->map_request_y != 0;
+}
+
 static bool
 weston_wm_window_type_inactive(struct weston_wm_window *window)
 {
@@ -2509,9 +2705,17 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 		wm->server->compositor->xwayland;
 	const struct weston_desktop_xwayland_interface *xwayland_interface =
 		wm->server->compositor->xwayland_interface;
-	struct weston_output *output;
 	struct weston_wm_window *parent;
 
+	/* This should be necessary only for override-redirected windows,
+	 * because otherwise MapRequest handler would have already updated
+	 * the properties. However, if X11 clients set properties after
+	 * sending MapWindow, here we can still process them. The decorations
+	 * have already been drawn once with the old property values, so if the
+	 * app changes something affecting decor after MapWindow, we glitch.
+	 * We only hit xserver_map_shell_surface() once per MapWindow and
+	 * wl_surface, so better ensure we get the window type right.
+	 */
 	weston_wm_window_read_properties(window);
 
 	/* A weston_wm_window may have many different surfaces assigned
@@ -2525,8 +2729,6 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 	wl_signal_add(&window->surface->destroy_signal,
 		      &window->surface_destroy_listener);
 
-	weston_wm_window_schedule_repaint(window);
-
 	if (!xwayland_interface)
 		return;
 
@@ -2542,6 +2744,9 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 						   window->surface,
 						   &shell_client);
 
+	wm_log("XWM: map shell surface, win %d, weston_surface %p, xwayland surface %p\n",
+	       window->id, window->surface, window->shsurf);
+
 	if (window->name)
 		xwayland_interface->set_title(window->shsurf, window->name);
 	if (window->pid > 0)
@@ -2550,11 +2755,9 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 	if (window->fullscreen) {
 		window->saved_width = window->width;
 		window->saved_height = window->height;
-		xwayland_interface->set_fullscreen(window->shsurf, NULL);
+		xwayland_interface->set_fullscreen(window->shsurf,
+						   window->legacy_fullscreen_output.output);
 		return;
-	} else if (legacy_fullscreen(wm, window, &output)) {
-		window->fullscreen = 1;
-		xwayland_interface->set_fullscreen(window->shsurf, output);
 	} else if (window->override_redirect) {
 		xwayland_interface->set_xwayland(window->shsurf,
 						 window->x, window->y);
@@ -2577,10 +2780,22 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 			xwayland_interface->set_xwayland(window->shsurf,
 							 window->x,
 							 window->y);
+		} else if (weston_wm_window_is_positioned(window)) {
+			xwayland_interface->set_toplevel_with_position(window->shsurf,
+								       window->map_request_x,
+								       window->map_request_y);
 		} else {
 			xwayland_interface->set_toplevel(window->shsurf);
 		}
 	}
+
+	if (window->frame_id == XCB_WINDOW_NONE) {
+		weston_wm_window_set_pending_state_OR(window);
+	} else {
+		weston_wm_window_set_pending_state(window);
+		weston_wm_window_set_allow_commits(window, true);
+		xcb_flush(wm->conn);
+	}
 }
 
 const struct weston_xwayland_surface_api surface_api = {
